; EVENT.ASM
; ------------------------------------------------------------
; Event handler called by the mouse driver. In turn calls the
; C event processor. Function can be compiled with TASM or
; MASM.
;
; MASM - MASM /MX EVENT
; TASM - TASM /MX EVENT
;-------------------------------------------------------------

.MODEL HUGE                   ; Change model to match C cmplr
.DATA			      ; Data directive
.CODE			      ; Code directive

PUBLIC _event_handler	      ; Handler is public for C code
EXTRN  _event_processor:near  ; The C event processor

eactive db 0		      ; EventActive flag so recursion
			      ; will not occur

;Module : EVENT HANDLER
;----------------------------
;
_event_handler PROC FAR	      ; MUST be FAR procedure

			      ; STACK MAINTENENCE
			      ;-------------------------
   push	  bp		      ; Push BP register
   mov	  bp,sp	              ; Assign SP to BP, stacks base
   sub	  sp,8 	              ; Subtract 8 from stack pointer
			      ; so args are available

			      ; DATA SEG MAINTENENCE
			      ;-------------------------
   push   ds		      ; Push driver data seg
   push   ax		      ; Push AX
   mov	  ax,DGROUP	      ; Move correct segment
   mov	  ds,ax	              ; Assign new segment

			      ; RECURSION CHECK
			      ;-------------------------
   cmp	  eactive,0	      ; Is event active now?
   jne	  norecurse	      ; YES - Go to end
			      ;-------------------------
   mov	  eactive,1	      ; Event is now active

			      ; EVENTS AND STATUS
			      ;-------------------------
			      ; Since AX was pushed already
   pop	  ax		      ; pop it for event status
   push	  dx		      ; Push Y coordinate
   push   cx		      ; Push X coordinate
   push   bx		      ; Push Button Status
   push   ax		      ; Push Event Status

			      ;-------------------------
   call   _event_processor    ; CALL C EVENT PROCESSOR!!
			      ;-------------------------
   add	  sp,8		      ; Update stack pointer
			      ; Arguments OFF stack


   mov	  eactive,0	      ; Event not active now
   jmp	  normend	      ; Goto normal end label

			      ; CLEANUP AND LEAVE
			      ;-------------------------
   norecurse:		      ; Recursive jump label
   pop	   ax		      ; AX didn't get popped if
			      ; event was already active

   normend:		      ; Normal end, AX already popped
   pop	   ds		      ; Pop data segment
   mov	   sp,bp	      ; Update stack pointer
   pop	   bp		      ; Pop BP register

   ret			      ; Exit

_event_handler ENDP	      ; End event handler
END			      ; End code
