unit Mouse;

{
  This mouse programming unit was translated from C-code from
  the book 'Building a better Mousetrap" (Jeffrey S. Donovan).
  It provides ten basic mousefunctions you'll always need, an
  event processor with an event handler in assembler, and six
  optional mouse functions.
}

interface

uses
  Dos;

{$I SHAPES.INC}

const
  Number_Buttons: integer = 2;
  HCell_Size: integer = 8;
  VCell_Size: integer = 8;
  Mouse_Size: integer = 16;
  CMX: integer = 0;
  CMY: integer = 0;
  BState: integer = 0;
  EGA_reg_Read: boolean = true;

var
  Mv_LButton, Mv_RButton, Mv_BButton: integer;
  In_GMode, Mouse_There, Mouse_Visible: boolean;
  LButton_Down,
  RButton_Down,
  BButton_Down,
  MButton_Down,
  LButton_Up,
  RButton_Up,
  BButton_Up,
  MButton_Up,
  Cursor_Moved: boolean;
  Regs: Registers;

procedure Reset_Mouse;
procedure Show_Mouse_Cursor;
procedure Hide_Mouse_Cursor;
procedure Set_Hide_Bounds(x1, y1, x2, y2: integer);
procedure Set_Mouse_Position(x, y: integer);
procedure Set_Mouse_HLimits(x1, x2: integer);
procedure Set_Mouse_VLimits(y1, y2: integer);
procedure Get_Mouse_Button(var LButton, RButton, MButton: boolean; var x, y: integer);
function Get_Button_RLS_Info(Butt_No: integer; var Count, x, y: integer): boolean;
procedure Set_Event_Handler(Call_Mask: integer; Ofset, Segment: word);

procedure Touch_Cursor;

procedure Install_Event_Handler;
procedure Reset_Event_Status;

procedure Get_Mouse_Type(var Maj_Ver, Min_Ver, MType, IRQ_Num: integer);
function Get_CRT_Page: integer;
procedure Get_Mouse_Sensitivity(var H_Speed, V_Speed, Treshold: integer);
procedure Set_Mouse_Sensitivity(H_Speed, V_Speed, Treshold: integer);
procedure Set_Graphics_Cursor(Cursor: GCursor);
procedure Set_Text_Cursor(CType, Sm_or_Sl, Cm_or_El: integer);

implementation

procedure Reset_Mouse;

begin
  Mouse_There:=false;
  Mouse_Size:=16;
  HCell_Size:=8;
  VCell_Size:=8;
  Mouse_Visible:=false;

  Regs.AX:=0;
  Intr($33, Regs);
  If Regs.AX <> 0 then Mouse_There:=true;

  Number_Buttons:=Regs.BX;

  If not In_GMode then
    begin
      Regs.AH:=$0F;
      Intr($10, Regs);
      Case Regs.AL of
        0: ;
        1: HCell_Size:=16
      end;
      Mouse_Size:=1;
    end;

  LButton_Down:=false;
  RButton_Down:=false;
  BButton_Down:=false
end;

procedure Show_Mouse_Cursor;

begin
  If Mouse_There then
    begin
      Regs.AX:=1;
      Intr($33, Regs);
      Mouse_Visible:=true
    end
end;

procedure Hide_Mouse_Cursor;

begin
  If (Mouse_Visible) and (Mouse_There) then
    begin
      Regs.AX:=2;
      Intr($33, Regs);
      Mouse_Visible:=false
    end
end;

procedure Set_Hide_Bounds(x1, y1, x2, y2: integer);

begin
  If (EGA_Reg_Read) and (Mouse_There) then
    begin
      If not In_GMode then
        begin
          x1:=(x1 * HCell_Size) - HCell_Size;
          x2:=(x2 * HCell_Size) - HCell_Size;
          y1:=(y1 * VCell_Size) - VCell_Size;
          y2:=(y2 * VCell_Size) - VCell_Size
        end;
      Regs.AX:=16;
      Regs.CX:=x1;
      Regs.DX:=y1;
      Regs.SI:=x2;
      Regs.DI:=y2;
      Intr($33, Regs)
    end
  else Hide_Mouse_Cursor
end;

procedure Set_Mouse_Position(x, y: integer);

begin
  If Mouse_There then
    begin
      If not In_GMode then
        begin
          x:=(x * HCell_Size) - HCell_Size;
          y:=(y * VCell_Size) - VCell_Size
        end;
      Regs.AX:=4;
      Regs.CX:=x;
      Regs.DX:=y;
      Intr($33, Regs)
    end
end;

procedure Set_Mouse_HLimits(x1, x2: integer);

begin
  If Mouse_There then
    begin
      If not In_GMode then
        begin
          x1:=(x1 * HCell_Size) - HCell_Size;
          x2:=(x2 * HCell_Size) - HCell_Size
        end;
      Regs.AX:=7;
      Regs.CX:=x1;
      Regs.DX:=x2;
      Intr($33, Regs)
    end
end;

procedure Set_Mouse_VLimits(y1, y2: integer);

begin
  If Mouse_There then
    begin
      If not In_GMode then
        begin
          y1:=(y1 * VCell_Size) - VCell_Size;
          y2:=(y2 * VCell_Size) - VCell_Size
        end;
      Regs.AX:=8;
      Regs.CX:=y1;
      Regs.DX:=y2;
      Intr($33, Regs)
    end
end;

procedure Get_Mouse_Button(var LButton, RButton, MButton: boolean; var x, y: integer);

begin
  If Mouse_There then
    begin
      Regs.AX:=3;
      Intr($33, Regs);

      If Regs.BX = 1 then LButton:=true else LButton:=false;
      If Regs.BX = 2 then RButton:=true else RButton:=false;
      If Regs.BX = 3 then
        begin
          LButton:=true;
          RButton:=true
        end;
      If Regs.BX = 4 then MButton:=true else MButton:=false;

      If In_GMode then
        begin
          x:=Regs.CX;
          y:=Regs.DX
        end
      else
        begin
          x:=(Regs.CX + HCell_Size) div HCell_Size;
          y:=(Regs.DX + VCell_Size) div VCell_Size
        end
    end
end;

function Get_Button_RLS_Info(Butt_No: integer; var Count, x, y: integer): boolean;

begin
  If Mouse_There then
    begin
      Regs.AX:=6;
      Regs.BX:=Butt_No;
      Intr($33, Regs);

      Count:=Regs.BX;
      x:=Regs.CX;
      y:=Regs.DX;
      If (Butt_No = 0) and ((Regs.AX AND 1) = 1) then Get_Button_RLS_Info:=true;
      If (Butt_No = 0) and ((Regs.AX AND 1) = 0) then Get_Button_RLS_Info:=false;
      If (Butt_No = 1) and ((Regs.AX AND 2) = 1) then Get_Button_RLS_Info:=true;
      If (Butt_No = 1) and ((Regs.AX AND 2) = 0) then Get_Button_RLS_Info:=false;
      If (Butt_No = 2) and ((Regs.AX AND 4) = 1) then Get_Button_RLS_Info:=true;
      If (Butt_No = 2) and ((Regs.AX AND 4) = 0) then Get_Button_RLS_Info:=false

    end
end;

procedure Set_Event_Handler(Call_Mask: integer; Ofset, Segment: word);

var
  Temp: integer;

begin
  If Mouse_There then
    begin
      Regs.AX:=12;
      Regs.CX:=Call_Mask;
      Regs.DX:=Ofset;
      Regs.ES:=Segment;
      Intr($33, Regs)
    end
end;

procedure Touch_Cursor;

var
  LButton, RButton, MButton: boolean;
  X, Y: integer;

begin
  Get_Mouse_Button(LButton, RButton, MButton, X, Y);
  Set_Mouse_Position(X, Y)
end;

procedure _Event_Handler; external; {$L EVENT.OBJ}

procedure _Event_Processor(y, x, Button_Status, Event_Status: integer);

begin
  If (CMX <> x) or (CMY <> y) then
    begin
      If not In_GMode then
        begin
          x:=(x + HCell_Size) div HCell_Size;
          y:=(y + VCell_Size) div VCell_Size
        end;
      Cursor_Moved:=true;
      CMX:=x;
      CMY:=y
    end;
  BState:=Button_Status;
  If Event_Status = 2 then LButton_Down:=true;
  If Event_Status = 8 then RButton_Down:=true;
  If (Number_Buttons = 3) and (Event_Status = 32) then MButton_Down:=true;
  If Event_Status = 10 then BButton_Down:=true;
  If Event_Status = 4 then LButton_Up:=true;
  If Event_Status = 16 then RButton_Up:=true;
  If (Number_Buttons = 3) and (Event_Status = 64) then MButton_Up:=true;
  If Event_Status = 20 then BButton_Up:=true
end;

procedure Install_Event_Handler;

begin
  If (Number_Buttons = 3) then Set_Event_Handler(127, Ofs(_Event_Handler), Seg(_Event_Handler))
    else Set_Event_Handler(31, Ofs(_Event_Handler), Seg(_Event_Handler));
end;

procedure Reset_Event_Status;

begin
  Cursor_Moved:=false;
  LButton_Down:=false;
  RButton_Down:=false;
  BButton_Down:=false;
  LButton_Up:=false;
  RButton_Up:=false;
  BButton_Up:=false
end;

procedure Get_Mouse_Type(var Maj_Ver, Min_Ver, MType, IRQ_Num: integer);

begin
  If Mouse_There then
    begin
      Regs.AX:=36;
      Intr($33, Regs);
      Maj_Ver:=Regs.BH;
      Min_Ver:=Regs.BL;
      MType:=Regs.CH;
      IRQ_Num:=Regs.CL
    end
end;

function Get_CRT_Page: integer;

begin
  If Mouse_There then
    begin
      Regs.AX:=30;
      Intr($33, Regs);
      Get_CRT_Page:=Regs.BX
    end
end;

procedure Get_Mouse_Sensitivity(var H_Speed, V_Speed, Treshold: integer);

begin
  If Mouse_There then
    begin
      Regs.AX:=27;
      Intr($33, Regs);
      H_Speed:=Regs.BX;
      V_Speed:=Regs.CX;
      Treshold:=Regs.DX
    end
end;

procedure Set_Mouse_Sensitivity(H_Speed, V_Speed, Treshold: integer);

begin
  If Mouse_There then
    begin
      If H_Speed <= 0 then H_Speed:=1;
      If V_Speed <= 0 then V_Speed:=1;
      If Treshold <= 0 then Treshold:=1;
      Regs.AX:=26;
      Regs.BX:=H_Speed;
      Regs.CX:=V_Speed;
      Regs.DX:=Treshold;
      Intr($33, Regs)
    end
end;

procedure Set_Graphics_Cursor(Cursor: GCursor);

begin
  If Mouse_There then
    begin
      Regs.AX:=9;
      Regs.BX:=Cursor.HotX;
      Regs.CX:=Cursor.HotY;
      Regs.DX:=Ofs(Cursor.Mask);
      Regs.ES:=Seg(Cursor.Mask);
      Intr($33, Regs)
    end
end;

procedure Set_Text_Cursor(CType, Sm_or_Sl, Cm_or_El: integer);

begin
  If Mouse_There then
    begin
      Regs.AX:=10;
      Regs.BX:=CType;
      Regs.CX:=Sm_or_Sl;
      Regs.DX:=Cm_or_El;
      Intr($33, Regs)
    end
end;

end.